<?php

/*
 * Copyright (C) 2010 Julien VAUBOURG / Sébastien BADIA
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

abstract class ActiveRecord {
	private $table; // Nom de la table SQL correspondante (<=> nom de la classe qui hérite avec le préfixe SQL)
	private $columns; // Tableau des valeurs des champs de la ligne de la table concernée (syntaxe: nom_colonne => valeur)

	/** Construction à partir de l'identifiant de la ligne concernée dans la table */
	public function __construct($id) {
		$this->class = strtolower(get_called_class());
		$this->table = TABLE_PREFIX.$this->class;

		// -1 permet d'instancier sans déclencher la lecture de la base
		// Utile pour ajouter les noms les champs manuellement (voir fake())
		if($id > -1)

			// Import des données depuis la base de données
			$this->read($id);
	}

	/** Getter automatique */
	public function __get($column) {

		// Les valeurs des champs de la base sont stockés dans le tableau columns
		return $column == "table" || $column == "class" || $column == "columns" ?
			$this->$column :
			$this->columns[$column];
	}

	/** Setter automatique */
	public function __set($column, $value) {
		if($column == "table") {
			if(empty($this->table))
				$this->table = $value;
		} elseif($column == "class")
			$this->class = $value;
		else
			$this->columns[$column] = $value;
	}

	/** Import de la valeur des différentes colonnes de la table correspondant au nom de la classe en cours
	en attribuant un attribut de la classe par colonne. */
	private function read($id) {

		// Sélection des champs
		$row = Mysql::get()->query("SELECT * FROM $this->table WHERE id = $id LIMIT 1");
		$columns = $row->fetch(PDO::FETCH_ASSOC);

		// Si la sélection est vide, cela signifie que l'enregistrement correspondant à l'identifiant donné
		// n'existe pas dans la table
		if(empty($columns))
			throw new Microsoft("Cet enregistrement de $this->class n'existe pas.");

		// 
		foreach($columns AS $name => $value)
			$this->$name = new Column($name, $value);
	}

	/** Synchronise les valeurs éventuellement modifiées de l'instance avec les valeurs des champs de la ligne de
	de la table */
	private function write() {
		$updates = null;

		// Pour chacune des valeurs des champs
		foreach($this->columns AS $col_name => $column) {

			// Si la valeur est de type de Column, cela signifie que c'est une valeur simple.
			// Le type Column propose deux attributs intuitifs : name et value
			if(get_class($column) == "Column") {

				// Le tableau updates[] ne stocke que les associations nom-valeur des champs qui ont
				// évolués depuis la lecture de la base de données
				if($column->isToUpdate) {
					$value = Mysql::get()->quote($column->value);
					$updates[] = "$column->name=$value";
					$column->isToUpdate = false;
				}

			// Si la valeur n'est pas un Column, c'est que c'est un autre objet qui représente donc le lien
			// avec une clé étrangére. C'est donc l'instance d'une autre entrée, et c'est son identifiant à elle
			// qui représente le lien à conserver pour ce champs dans la base.
			} else {
				// FIXME: update dans tous les cas
				$updates[] = $column->class."=".$column->id->value;

				// Les instances liées ont pu aussi être mises à jour par le biais de celle-ci, on force donc la
				// synchro. au même moment.
				$column->write();
			}
		}

		// Mises à jour SQL si nécessaire
		if(!is_null($updates)) {
			$update = implode(",", $updates);
			Mysql::get()->query("UPDATE $this->table SET $update WHERE id = {$this->id->value}");
		}
	}

	/** Affichage HTML de l'instance à partir du fichier template correspondant */
	protected function display($template) {

		// Récupération du template
		$tpl = file_get_contents($template);

		// Remplacement des principaux éléments
		$tpl = str_replace("<TPL:ID>", $this->id->value, $tpl);
		$tpl = str_replace("<TPL:CLASSE>", $this->class, $tpl);

		// Remplacement de tous les tags <TPL:NOM_COLONNE> par la valeur correspondante
		foreach($this->columns AS $col_name => $column) {
			$tplColumn = strtoupper($col_name);
			$tpl = str_replace("<TPL:$tplColumn>", $column, $tpl);
		}

		return $tpl;
	}

	/** Affichage HTML de l'instance, dans le cadre d'un listage d'une série d'elles */
	public function displayForListing() {
		return $this->display("templates/{$this->class}_listItem.tpl");
	}

	/** Fait ajouter une ligne dans la table et renvoie une instance de cette nouvelle ligne.
	Utilise newRow(). */
	abstract public static function add();

	/** Création d'une fausse instance n'ayant aucune correspondance dans la base de donnée.
	Sert principalement pour utiliser la fonction d'affichage. */
	public static function fake() {
		$table = TABLE_PREFIX.strtolower(get_called_class());
		$class = ucfirst(get_called_class());

		// Récupére les noms des champs de la table
		$columns = Mysql::get()->query("SHOW COLUMNS FROM ".$table);

		// Instanciation d'une instance sans lecture de la base de données (donc sans valeur)
		$fake = new $class(-1);

		// Pour chacun des champs, on les crée dans l'instance sans valeur
		foreach($columns->fetchAll(PDO::FETCH_OBJ) AS $column) {
			$field = $column->Field;
			$fake->$field = new Column($field);
		}

		return $fake;
	}

	/** Ajoute une ligne vide dans la table SQL et renvoie l'identifiant de cette nouvelle ligne  */
	protected static function newRow() {
		$table = TABLE_PREFIX.strtolower(get_called_class());

		Mysql::get()->query("INSERT INTO $table VALUES()");
		$newId = Mysql::get()->query("SELECT LAST_INSERT_ID() FROM $table");
		$newId = $newId->fetch(PDO::FETCH_COLUMN);

		return $newId;
	}

	/** Liste toutes les lignes de la table correspondante à la classe, en les instanciant un à un
	et les affichant sous forme de liste via leur displayForListing. */
	public static function toList($where = null) {
		$class = strtolower(get_called_class());
		$table = TABLE_PREFIX.$class;
		$theClass = ucfirst($class);
		$html_list = null;

		try {
			// Sélectionne toutes les lignes de la table, en modérant éventuellement avec le $where
			$list = Mysql::get()->query("SELECT id FROM $table $where ORDER BY id ASC");
			$tpl_list = file_get_contents("templates/{$class}_list.tpl");

			// Pour chacune, on instancie et on affiche
			while($idItem = $list->fetch(PDO::FETCH_COLUMN)) {
				$item = new $theClass($idItem);
				$html_list .= $item->displayForListing();
			}

		} catch(PDOException $e) {}

		// Si la table est vide ou si le $where a trop restreint
		$html_list = empty($html_list) ? "Aucun élément." : $html_list;

		return str_replace("<TPL:LISTE>", $html_list, $tpl_list);
	}

	/** Supprimer une ligne de la table correspondante à la classe à partir d'un id */
	public static function delete($id) {
		$table = TABLE_PREFIX.strtolower(get_called_class());
		Mysql::get()->query("DELETE FROM $table WHERE id=".intval($id));
	}

	public function __toString() {
		return $this->display("templates/$this->class.tpl");
	}

	/** Si la table a un identifiant (dans tous les cas sauf si on instancie avec -1 sans forcer
	la lecture), on force la synchro. avec la base quand l'instance se détruit. */
	public function __destruct() {
		if(!is_null($this->id->value))
			$this->write();
	}
}

?>
