<?php

/*
 * Copyright (C) 2010 Julien VAUBOURG / Sébastien BADIA
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

class Page extends ActiveRecord {

	// @Override
	/** Ajoute la gestion de la clé étrangére, éventuellement désactivable (par ex. dans le cas
	d'une désactivation de la lecture dans la BDD avec l'id -1 ou dans le cas d'un ajout dans la
	table. */
	public function __construct($id, $initForeignKeys = true) {
		parent::__construct($id);

		// Instanciation de l'enregistrement correspondant à la clé étrangére plutôt que de
		// stocker l'id.
		if($id > -1 && $initForeignKeys)
			$this->entree = new Entree($this->entree->value);
	}

	/** Affiche un formulaire de modification de l'instance à partir d'un template */
	public function edit() {
		$tpl = file_get_contents("templates/page_edit.tpl");

		$tpl = str_replace("<TPL:TEXTE>", $this->texte, $tpl);
		$tpl = str_replace("<TPL:NOM>", urldecode($this->entree->nom), $tpl);

		return $tpl;
	}

	// @Abstract
	/** Ajoute une entrée dans la BDD en précisant éventuellement les clés étrangéres associées */
	public static function add($idAuteur = null, $idEntree = null) {

		// Ajout dans la base d'une nouvelle ligne et instanciation de cette ligne
		$page = new Page(Page::newRow(), false);

		// Si un id est proposé pour la clé étrangére <entree>, on l'instancie, sinon
		// on demande son ajout (cas de création d'une nouvelle page et pas juste d'une
		// version).
		$page->entree = is_null($idEntree) ?
			Entree::add($idAuteur) :
			new Entree($idEntree);

		return $page;
	}

	// @Override
	/** Instanciation d'une fausse instance, ajout de la fausse instanciation de la clé étrangère */
	public static function fake() {
		$fake = parent::fake();
		$fake->entree = Entree::fake();

		return $fake;
	}

	/** Affiche les différences avec une autre instance de Page, qui concerne la même <entree>
	et qui est donc juste une version différente de celle-ci. */
	public function diffWith($page) {

		// Ecriture des fichiers contenant chacun une version du texte
		file_put_contents("config/diff1", $this->texte->value);
		file_put_contents("config/diff2", $page->texte->value);

		// Comparaison des fichiers avec diff qui renverra les différences sous une syntaxe particulière
		exec("diff config/diff1 config/diff2", $diff);

		// Suppression des fichies
		unlink("config/diff1");
		unlink("config/diff2");

		// Pour chacune des lignes du retour de la commande diff, on la transforme en HTML et on
		// les fusionne
		for($i = 0; $i < count($diff); $diff[$i] = formatDiffLine($diff[$i]), $i++);
		$diff = implode("\n", $diff);

		return $diff;
	}

	/** Renvoie l'identifiant de la page qui correspondant à la derniere version d'une entree */
	public static function idLastVersion($idEntree_or_nom) {

		// Si le paramére passé n'est pas un nombre, c'est qu'on passe le nom de l'entrée.
		// On trouve donc l'identifiant correspondant.
		if(!is_numeric($idEntree_or_nom)) {
			$nom = $idEntree_or_nom;
			$entrees = Mysql::get()->query("SELECT id FROM ".TABLE_PREFIX."entree WHERE nom=".Mysql::get()->quote($nom)." LIMIT 1");
			$idEntree_or_nom = $entrees->fetch(PDO::FETCH_COLUMN);
		}

		// Seul l'identifiant sera utilisé
		$idEntree = $idEntree_or_nom;

		// Récupération de l'id de la page la plus récente qui concerne l'entrée
		if(!empty($idEntree)) {
			$versions = Mysql::get()->query("SELECT id FROM ".TABLE_PREFIX."page WHERE entree=$idEntree ORDER BY datemodif DESC LIMIT 1");
			$idCurVersion = $versions->fetch(PDO::FETCH_COLUMN);
		} else
			throw new Microsoft("L'Entree n'existe pas.");

		return $idCurVersion;
	}

	private function replaceEditRestoreLink($tpl) {
		$editer_retablir = null;

		// Si la page n'est pas la dernière version d'une entree, on affiche en gros la date de la version
		$isLastVersion = (Page::idLastVersion($this->entree->id->value) == $this->id->value);
		$tpl = str_replace("<TPL:VERSION>", !$isLastVersion ? '<h3 id="ancienneVersion">Version du '.truncDate($this->datemodif->value, 2)."</h3>" : NULL, $tpl);

		// Si l'utilisateur connecté est le créateur de la page ou que c'est l'admin et que ça n'est pas la dernière version, on ajoute un bouton proposant de
		// rétablir la version.
		if((USER == $this->entree->auteur->id->value || ADMIN) && !$isLastVersion)
			$editer_retablir = '<a href="restore.php?nom='.$this->entree->nom->value."&amp;vdate=".urlencode($this->datemodif->value).'" class="bouton">Rétablir</a>';

		// Si l'utilisateur est juste connecté et que c'est la dernière version, on ajoute un bouton d'édition de la page (d'ajout de version)
		elseif(CONNECTED && $isLastVersion)
			$editer_retablir = '<a href="edit.php?nom='.$this->entree->nom->value.'" class="bouton">Editer</a>';
		else
			$editer_retablir = null;

		return str_replace("<TPL:EDITER_RETABLIR>", $editer_retablir, $tpl);
	}

	public function __toString() {
		$tpl = parent::__toString();

		try {
			$tpl = $this->replaceEditRestoreLink($tpl);

			// Affichage de la liste de commentaires associés
			$tpl = str_replace("<TPL:COMMENTAIRES>", str_replace("Aucun élément.", null, Commentaire::toList("WHERE entree={$this->entree->id->value}")), $tpl);

		} catch(Microsoft $e) {
			$tpl = $e->__toString();
		}

		return $tpl;
	}
}

?>
